@@ -12,76 +12,107 @@ require AutoLoader;
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
-@EXPORT = qw
-  (
-   PVM_BYTE PVM_CPLX PVM_DCPLX PVM_DOUBLE PVM_FLOAT PVM_INT
-   PVM_LONG PVM_SHORT PVM_STR PVM_UINT PVM_ULONG PVM_USHORT
-   PvmAllowDirect PvmAlready PvmAutoErr PvmBadMsg PvmBadParam
-   PvmBadVersion PvmCantStart PvmDSysErr PvmDataDefault
-   PvmDataFoo PvmDataInPlace PvmDataRaw PvmDebugMask PvmDontRoute
-   PvmDupEntry PvmDupGroup PvmDupHost PvmFragSize PvmHostAdd
-   PvmHostCompl PvmHostDelete PvmHostFail PvmMismatch PvmMppFront
-   PvmNoBuf PvmNoData PvmNoEntry PvmNoFile PvmNoGroup PvmNoHost
-   PvmNoInst PvmNoMem PvmNoParent PvmNoSuchBuf PvmNoTask
-   PvmNotImpl PvmNotInGroup PvmNullGroup PvmOk PvmOutOfRes
-   PvmOutputCode PvmOutputTid PvmOverflow PvmPollConstant
-   PvmPollSleep PvmPollTime PvmPollType PvmResvTids PvmRoute
-   PvmRouteDirect PvmSelfOutputCode PvmSelfOutputTid
-   PvmSelfTraceCode PvmSelfTraceTid PvmShowTids PvmSysErr
-   PvmTaskArch PvmTaskChild PvmTaskDebug PvmTaskDefault
-   PvmTaskExit PvmTaskHost PvmTaskSelf PvmTaskTrace PvmTraceCode
-   PvmTraceTid PvmMboxDefault PvmMboxPersistent PvmMboxMultiInstance 
-   PvmMboxOverWritable PvmMboxFirstAvail PvmMboxReadAndDelete
+@EXPORT = qw(
+	PVM_BYTE
+	PVM_CPLX
+	PVM_DCPLX
+	PVM_DOUBLE
+	PVM_FLOAT
+	PVM_INT
+	PVM_LONG
+	PVM_SHORT
+	PVM_STR
+	PVM_UINT
+	PVM_ULONG
+	PVM_USHORT
+	PvmAllowDirect
+	PvmAlready
+	PvmAutoErr
+	PvmBadMsg
+	PvmBadParam
+	PvmBadVersion
+	PvmCantStart
+	PvmDSysErr
+	PvmDataDefault
+	PvmDataFoo
+	PvmDataInPlace
+	PvmDataRaw
+	PvmDebugMask
+	PvmDontRoute
+	PvmDupEntry
+	PvmDupGroup
+	PvmDupHost
+	PvmFragSize
+	PvmHostAdd
+	PvmHostCompl
+	PvmHostDelete
+	PvmHostFail
+	PvmMismatch
+	PvmMppFront
+	PvmNoBuf
+	PvmNoData
+	PvmNoEntry
+	PvmNoFile
+	PvmNoGroup
+	PvmNoHost
+	PvmNoInst
+	PvmNoMem
+	PvmNoParent
+	PvmNoSuchBuf
+	PvmNoTask
+	PvmNotImpl
+	PvmNotInGroup
+	PvmNullGroup
+	PvmOk
+	PvmOutOfRes
+	PvmOutputCode
+	PvmOutputTid
+	PvmOverflow
+	PvmPollConstant
+	PvmPollSleep
+	PvmPollTime
+	PvmPollType
+	PvmResvTids
+	PvmRoute
+	PvmRouteDirect
+	PvmSelfOutputCode
+	PvmSelfOutputTid
+	PvmSelfTraceCode
+	PvmSelfTraceTid
+	PvmShowTids
+	PvmSysErr
+	PvmTaskArch
+	PvmTaskChild
+	PvmTaskDebug
+	PvmTaskDefault
+	PvmTaskExit
+	PvmTaskHost
+	PvmTaskSelf
+	PvmTaskTrace
+	PvmTraceCode
+	PvmTraceTid
 );
 
-# Theese are the badd ones:
-#     send pack unpack exit recv kill 
-@EXPORT_OK = qw
-  (
+$VERSION = '1.1';
 
-   spawn initsend psend mcast sendsig probe nrecv trecv precv parent
-   mytid halt catchout tasks config addhosts delhosts bufinfo freebuf
-   getrbuf getsbuf mkbuf setrbuf setsbuf mstat pstat tidtohost getopt
-   setopt reg_hoster reg_tasker reg_rm perror notify recv_notify
-   hostsync recvf recvf_old
-
-   joingroup lvgroup bcast freezegroup barrier getinst gettid gsize
-
-   siblings
-
-   getcontext newcontext setcontext freecontext
-
-   putinfo recvinfo delinfo getmboxinfo
-
-   code2symbol code2text
-  );
-
-$VERSION = '1.4.0-pre1';
-
-sub AUTOLOAD 
-{
+sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
     # to the AUTOLOAD in AutoLoader.
 
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
-    croak "& not defined" if $constname eq 'constant';
     my $val = constant($constname, @_ ? $_[0] : 0);
-    if ($! != 0) 
-    {
-	if ($! =~ /Invalid/) 
-        {
+    if ($! != 0) {
+	if ($! =~ /Invalid/) {
 	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
 	    goto &AutoLoader::AUTOLOAD;
 	}
-	else 
-        {
+	else {
 		croak "Your vendor has not defined Parallel::Pvm macro $constname";
 	}
     }
-    no strict 'refs';
-    *$AUTOLOAD = sub { $val };
+    eval "sub $AUTOLOAD { $val }";
     goto &$AUTOLOAD;
 }
 
@@ -89,7 +120,6 @@ bootstrap Parallel::Pvm $VERSION;
 
 # Preloaded methods go here.
 
-
 # Autoload methods go after =cut, and are processed by the autosplit program.
 
 1;
@@ -345,15 +375,15 @@ the example code sequence
 	Parallel::Pvm::pack(1234);
 	Parallel::Pvm::send($dtid,999);
 
-In our example we first call B<Parallel::Pvm::initsend> to initialize
-the internal PVM send buffer.  We then call B<Parallel::Pvm::buffer>
-to fill this buffer with a double (2.345), a string ("hello dude"),
-and an integer (1234) <b>Actually, currently all arguments are
-converted to strings</b>.  Having filled the send buffer with the data
-that is to be sent, we call B<Parallel::Pvm::send> to do the actual
-send to the task identifed by the B<TID> B<$dtid>.  We also label the
-sending message to disambiguate it with other messages with a tag.
-This is done with the 999 argument in B<Parallel::Pvm::send> function.
+In our example we first call B<Parallel::Pvm::initsend> to initialize 
+the internal PVM send buffer.  
+We then call B<Parallel::Pvm::buffer> to fill this buffer with a double (2.345),
+, a string ("hello dude"), and an integer (1234).  
+Having filled the send buffer with the data that is to be sent, 
+we call B<Parallel::Pvm::send> to do the actual send to the task identifed by the B<TID> 
+B<$dtid>.   We also label the sending message to disambiguate it with 
+other messages with a tag.  This is done with the 999 argument in 
+B<Parallel::Pvm::send> function.  
 
 For the destination task, we can receive the message sent by 
 performing a blocking receive with the function B<Parallel::Pvm::recv>.  
@@ -375,11 +405,6 @@ example,
 	$tag = 999;
 	Parallel::Pvm::recv($stid,$tag) ;
 
-I<Caveats>: Messages may not contain the vertical tab character
-C<"\v">. If you pass messages to programs written in other languages,
-you need to know that C<Parallel::Pvm::pack> packs everything as
-strings (with C<pvm_packstr>).
-
 Other message passing functions that you may find useful are 
 B<Parallel::Pvm::psend>, B<Parallel::Pvm::trecv>, B<Parallel::Pvm::nrecv> and B<Parallel::Pvm::precv>.  
 
@@ -505,28 +530,20 @@ B<Server:>
 
 =head2 PVM groups 
 
-The PVM dynamic group functions have not completely been ported to
-Perl yet.  We do not support B<pvm_scatter>, B<pvm_gather>, and
-B<pvm_reduce> currently.  This is connected to the limited datatype
-support in the rest of the Perl interface.
-
-The group functions provide facilities for collecting processes under
-a single B<group> label, and applying aggregate operations onto them.
-Examples of these functions are B<Parallel::Pvm::barrier>,
-B<Parallel::Pvm::reduce>, B<Parallel::Pvm::bcast> etc.  One of our
-concerns is that these group functions may be changed or augmented in
-the future releases of PVM 3.4*.
+The PVM dynamic group functions have not been ported to perl yet.  
+These functions provide facilities for collecting processes under 
+a single B<group> label, and applying aggregate operations onto 
+them.  Examples of these functions are B<Parallel::Pvm::barrier>, B<Parallel::Pvm::reduce>, 
+B<Parallel::Pvm::bcast> etc.  
+One of our concerns is that these group functions may be 
+changed or augmented in the future releases of PVM 3.4*. A decision 
+for porting the group functions will be made after 
+PVM 3.4 has been released.  
 
 =head1 FUNCTIONS
 
 =over 4
 
-=item B<Parallel::Pvm::start_pvmd>
-
-Starts pvmd if it's not already running.
-
-	$info = Parallel::Pvm::start_pvmd($block, @args) ;
-
 =item B<Parallel::Pvm::addhosts> 
 
 Adds one or more host names to a parallel virtual machine. Eg.
@@ -698,8 +715,7 @@ Receives a message directly into a buffer.
 
 	@recv_buffer = Parallel::Pvm::precv($tid,$tag);
 
-Note that the current limit for the receive buffer is 100 KBytes
-unless you specify a third argument overwriting this limit.
+Note that the current limit for the receive buffer is 100 KBytes.  
 
 =item B<Parallel::Pvm::probe>
 
@@ -865,98 +881,15 @@ Unpacks the active receive message buffer.  Eg.
 
 	@recv_buffer = Parallel::Pvm::unpack ;
 
-An optional integer argument gives the maximum message size to unpack.
-Default is 100_000 bytes.
-
 =back
 
-=head1 AUTHORS
+=head1 AUTHOR
 
 Edward Walker, edward@nsrc.nus.sg,
 National Supercomputing Research Centre, Singapore
 
-Denis Leconte, denis_leconte@geocities.com 
-
-Ulrich Pfeifer, pfeifer@wait.de
-
 =head1 SEE ALSO
 
 perl(1), pvm_intro(1PVM)
 
 =cut
-
-sub code2symbol ($ ) {
-  my %n2s = (
-             0    => 'Ok',
-             -2   => 'BadParam',
-             -3   => 'Mismatch',
-             -4   => 'Overflow',
-             -5   => 'NoData',
-             -6   => 'NoHost',
-             -7   => 'NoFile',
-             -8   => 'Denied',
-             -10  => 'NoMem',
-             -12  => 'BadMsg',
-             -14  => 'SysErr',
-             -15  => 'NoBuf',
-             -16  => 'NoSuchBuf',
-             -17  => 'NullGroup',
-             -18  => 'DupGroup',
-             -19  => 'NoGroup',
-             -20  => 'NotInGroup',
-             -21  => 'NoInst',
-             -22  => 'HostFail',
-             -23  => 'NoParent',
-             -24  => 'NotImpl',
-             -25  => 'DSysErr',
-             -26  => 'BadVersion',
-             -27  => 'OutOfRes',
-             -28  => 'DupHost',
-             -29  => 'CantStart',
-             -30  => 'Already',
-             -31  => 'NoTask',
-             -32  => 'NotFound',
-             -33  => 'Exists',
-             -34  => 'HostrNMstr',
-             -35  => 'ParentNotSet',
-            );
-  $n2s{$_[0]}
-}
-
-sub code2text ($ ) {
-  my %n2t = (
-             0   => "Success",
-             -2  => "Bad parameter",
-             -3  => "Parameter mismatch",
-             -4  => "Value too large",
-             -5  => "End of buffer",
-             -6  => "No such host",
-             -7  => "No such file",
-             -8  => "Permission denied",
-             -10 => "Malloc failed",
-             -12 => "Can't decode message",
-             -14 => "Can't contact local daemon",
-             -15 => "No current buffer",
-             -16 => "No such buffer",
-             -17 => "Null group name",
-             -18 => "Already in group",
-             -19 => "No such group",
-             -20 => "Not in group",
-             -21 => "No such instance",
-             -22 => "Host failed",
-             -23 => "No parent task",
-             -24 => "Not implemented",
-             -25 => "Pvmd system error",
-             -26 => "Version mismatch",
-             -27 => "Out of resources",
-             -28 => "Duplicate host",
-             -29 => "Can't start pvmd",
-             -30 => "Already in progress",
-             -31 => "No such task",
-             -32 => "Not Found",
-             -33 => "Already exists",
-             -34 => "Hoster run on non-master host",
-             -35 => "Spawning parent set PvmNoSpawnParent",
-            );
-  $n2t{$_[0]}
-}
